package socialnetworkservicelogic

import (
	"app-cloudep-tweeting-service/internal/domain"
	"app-cloudep-tweeting-service/internal/domain/repository"
	"context"

	ers "code.30cm.net/digimon/library-go/errs"

	"app-cloudep-tweeting-service/gen_result/pb/tweeting"
	"app-cloudep-tweeting-service/internal/svc"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetFollowerLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetFollowerLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetFollowerLogic {
	return &GetFollowerLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

type getFollowReq struct {
	UID       string `validate:"required"`
	PageSize  int64  `validate:"required"`
	PageIndex int64  `validate:"required"`
}

// GetFollower 取得跟隨者名單
func (l *GetFollowerLogic) GetFollower(in *tweeting.FollowReq) (*tweeting.FollowResp, error) {
	// 驗證資料
	if err := l.svcCtx.Validate.ValidateAll(&getFollowReq{
		UID:       in.Uid,
		PageSize:  in.PageSize,
		PageIndex: in.PageIndex,
	}); err != nil {
		// 錯誤代碼 05-011-00
		return nil, ers.InvalidFormat(err.Error())
	}

	follower, err := l.svcCtx.SocialNetworkRepository.GetFollower(l.ctx, repository.FollowReq{
		UID:       in.GetUid(),
		PageIndex: in.GetPageIndex(),
		PageSize:  in.GetPageSize(),
	})
	if err != nil {
		// 錯誤代碼 05-021-31
		e := domain.CommentErrorL(
			domain.GetFollowerErrorCode,
			logx.WithContext(l.ctx),
			[]logx.LogField{
				{Key: "req", Value: in},
				{Key: "func", Value: "SocialNetworkRepository.GetFollower"},
				{Key: "err", Value: err},
			},
			"failed to get relation: ", in.GetUid()).Wrap(err)

		return nil, e
	}

	return &tweeting.FollowResp{
		Uid: follower.UIDs,
		Page: &tweeting.Pager{
			Total: follower.Total,
			Index: in.GetPageIndex(),
			Size:  in.GetPageSize(),
		},
	}, nil
}
