package timelineservicelogic

import (
	"app-cloudep-tweeting-service/internal/domain"
	"context"

	ers "code.30cm.net/digimon/library-go/errs"

	"app-cloudep-tweeting-service/gen_result/pb/tweeting"
	"app-cloudep-tweeting-service/internal/svc"

	"github.com/zeromicro/go-zero/core/logx"
)

type ClearNoMoreDataFlagLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewClearNoMoreDataFlagLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ClearNoMoreDataFlagLogic {
	return &ClearNoMoreDataFlagLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

type clearNoMoreDataFlagReq struct {
	UID string `json:"uid" validate:"required"`
}

// ClearNoMoreDataFlag 清除時間線的 "NoMoreData" 標誌。
func (l *ClearNoMoreDataFlagLogic) ClearNoMoreDataFlag(in *tweeting.DoNoMoreDataReq) (*tweeting.OKResp, error) {
	// 驗證資料
	if err := l.svcCtx.Validate.ValidateAll(&clearNoMoreDataFlagReq{
		UID: in.GetUid(),
	}); err != nil {
		// 錯誤代碼 05-011-00
		return nil, ers.InvalidFormat(err.Error())
	}

	err := l.svcCtx.TimelineRepo.ClearNoMoreDataFlag(l.ctx, in.GetUid())
	if err != nil {
		// 錯誤代碼 05-021-22
		e := domain.CommentErrorL(
			domain.ClearNoMoreDataErrorCode,
			logx.WithContext(l.ctx),
			[]logx.LogField{
				{Key: "req", Value: in},
				{Key: "func", Value: "TimelineRepo.ClearNoMoreDataFlag"},
				{Key: "err", Value: err},
			},
			"failed to clear no more data flag timeline repo :", in.GetUid()).Wrap(err)

		return nil, e
	}

	return &tweeting.OKResp{}, nil
}
